@@ -1221,7 +1221,6 @@ extern int
 env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 			const char *node_name)
 {
-	bool ntasks_set;
 	char *tmp = NULL;
 	int i;
 	slurm_step_layout_t *step_layout = NULL;
@@ -1234,7 +1233,6 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	if (!batch)
 		return SLURM_ERROR;
 
-	ntasks_set = batch->ntasks ? true : false;
 	memset(&step_layout_req, 0, sizeof(slurm_step_layout_req_t));
 	step_layout_req.num_tasks = batch->ntasks;
 
@@ -1248,8 +1246,9 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 
 	/*
 	 * --ntasks-per-node no-longer sets num_tasks implicitly, so we need
-	 * need to calculate num_tasks here to make sure the environment
-	 * variable is correct.
+	 * need to calculate num_tasks here to make sure the
+	 * SLURM_TASKS_PER_NODE environment variable is correct. Also make sure
+	 * that the SLURM_NTASKS environment variable is set.
 	 *
 	 * --ntasks-per-tres still implicitly sets ntasks.
 	 * --ntasks-per-socket requires --ntasks in order to work.
@@ -1315,7 +1314,7 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 		env_array_overwrite_fmt(dest, "SLURM_CPUS_PER_TASK", "%u",
 					cpus_per_task);
 
-	if (ntasks_set) {
+	if (step_layout_req.num_tasks) {
 		env_array_overwrite_fmt(dest, "SLURM_NTASKS", "%u",
 					step_layout_req.num_tasks);
 		/* keep around for old scripts */
@@ -1999,7 +1998,7 @@ int env_array_to_file(const char *filename, const char **env_array,
 		/* skip any env variables with a newline in newline mode */
 		if (newline && xstrstr(*p, "\n")) {
 			log_flag_hex(STEPS, *p, strlen(*p),
-				     "%s: skiping environment variable with newline",
+				     "%s: skipping environment variable with newline",
 				     __func__);
 			continue;
 		}
