@@ -54,13 +54,13 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/proc_args.h"
 #include "src/common/parse_time.h"
 #include "src/common/slurm_protocol_api.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
@@ -439,6 +439,9 @@ uint64_t str_to_mbytes(const char *arg)
 	else
 		return NO_VAL64;
 
+	if (result < 0)
+		return NO_VAL64;
+
 	return (uint64_t) result;
 }
 
@@ -485,15 +488,20 @@ _str_to_nodes(const char *num_str, char **leftover)
 	}
 	*leftover = endptr;
 
+	if ((num < 0) || (num > INT_MAX))
+		return -1;
+
 	return (int)num;
 }
 
 /*
- * verify that a node count in arg is of a known form (count or min-max)
+ * verify that a node count in arg is of a known form (count or min-max or list)
  * OUT min, max specified minimum and maximum node counts
+ * OUT job_size_str
  * RET true if valid
  */
-bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes)
+bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes,
+		       char **job_size_str)
 {
 	char *ptr, *min_str, *max_str;
 	char *leftover;
@@ -502,7 +510,48 @@ bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes)
 	 * Does the string contain a "-" character?  If so, treat as a range.
 	 * otherwise treat as an absolute node count.
 	 */
-	if ((ptr = xstrchr(arg, '-')) != NULL) {
+	if (job_size_str)
+		xfree(*job_size_str);
+
+	if ((ptr = xstrchr(arg, ',')) || (ptr = xstrchr(arg, ':'))) {
+		bitstr_t *job_size_bitmap;
+		char *tok, *tmp_str, *save_ptr = NULL;
+		long int max = 0;
+
+		tmp_str = xstrdup(arg);
+
+		tok = strtok_r(tmp_str, ",-:", &save_ptr);
+		while (tok) {
+			char *endptr;
+			long int num = strtol(tok, &endptr, 10);
+			if ((endptr == tok) || ((*endptr != '\0') &&
+						(*endptr != ',') &&
+						(*endptr != '-') &&
+						(*endptr != ':'))) {
+				error("\"%s\" is not a valid node count", tok);
+				xfree(tmp_str);
+				return false;
+			}
+			if (num > max)
+				max = num;
+			tok = strtok_r(NULL, ",-:", &save_ptr);
+		}
+		xfree(tmp_str);
+		tmp_str = xstrdup(arg);
+		job_size_bitmap = bit_alloc(max + 1);
+		if (bit_unfmt(job_size_bitmap, tmp_str)) {
+			error("\"%s\" is not a valid node count", arg);
+			FREE_NULL_BITMAP(job_size_bitmap);
+			xfree(tmp_str);
+			return false;
+		}
+		*min_nodes = bit_ffs(job_size_bitmap);
+		*max_nodes = bit_fls(job_size_bitmap);
+		if (job_size_str)
+			*job_size_str = bit_fmt_full(job_size_bitmap);
+		FREE_NULL_BITMAP(job_size_bitmap);
+		xfree(tmp_str);
+	} else if ((ptr = xstrchr(arg, '-')) != NULL) {
 		min_str = xstrndup(arg, ptr-arg);
 		*min_nodes = _str_to_nodes(min_str, &leftover);
 		if (!xstring_is_whitespace(leftover)) {
@@ -1354,10 +1403,10 @@ extern int parse_int(const char *name, const char *val, bool positive)
 	    (positive && (result <= 0L))) {
 		error ("Invalid numeric value \"%s\" for %s.", val, name);
 		exit(1);
-	} else if (result == LONG_MAX) {
+	} else if (result >= INT_MAX) {
 		error ("Numeric argument (%ld) to big for %s.", result, name);
 		exit(1);
-	} else if (result == LONG_MIN) {
+	} else if (result <= INT_MIN) {
 		error ("Numeric argument (%ld) to small for %s.", result, name);
 		exit(1);
 	}
