@@ -211,16 +211,6 @@ extern int slurm_init_msg_engine_port(uint16_t port);
  */
 extern int slurm_init_msg_engine_ports(uint16_t *);
 
-/*
- * bind() and then listen() to any port in a given range of ports
- *
- * IN: s - socket
- * IN: port - port number to attempt to bind
- * IN: local - only bind to localhost if true
- * OUT: port was bound successfully or -1 on failure
- */
-extern int sock_bind_listen_range(int s, uint16_t *range, bool local);
-
 /* In the socket implementation it creates a socket, binds to it, and
  *	listens for connections.
  * IN slurm_address 	- slurm_addr_t to bind the msg server to
