@@ -52,10 +52,11 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/slurm_xlator.h"
+#include "src/common/assoc_mgr.h"
 #include "src/common/bitstring.h"
 #include "src/common/env.h"
-#include "src/common/gpu.h"
-#include "src/common/gres.h"
+#include "src/interfaces/gpu.h"
+#include "src/interfaces/gres.h"
 #include "src/common/list.h"
 #include "src/common/strnatcmp.h"
 #include "src/common/xstring.h"
@@ -90,9 +91,10 @@
 const char plugin_name[] = "Gres GPU plugin";
 const char	plugin_type[]		= "gres/gpu";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
-static char	*gres_name		= "gpu";
 static List	gres_devices		= NULL;
 static uint32_t	node_flags		= 0;
+static int tres_mem_pos = -1;
+static int tres_util_pos = -1;
 
 extern void gres_p_step_hardware_init(bitstr_t *usable_gpus, char *tres_freq)
 {
@@ -420,7 +422,8 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 	while ((gres_slurmd_conf = list_next(itr))) {
 		int i;
 		hostlist_t hl;
-		char *hl_name;
+		char *hl_name, *tmp_file;
+		int count = gres_slurmd_conf->count;
 
 		if (!gres_slurmd_conf->count)
 			continue;
@@ -440,21 +443,15 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			list_append(gres_list_conf_single, gres_slurmd_conf);
 			continue;
 		} else if (!gres_slurmd_conf->file) {
+			gres_slurmd_conf->count = 1;
 			/*
 			 * Split this record into multiple single-GPU records
 			 * and add them to the single-GPU GRES list
 			 */
-			for (i = 0; i < gres_slurmd_conf->count; i++)
+			for (i = 0; i < count; i++)
 				add_gres_to_list(gres_list_conf_single,
-						 gres_slurmd_conf->name, 1,
-						 gres_slurmd_conf->cpu_cnt,
-						 gres_slurmd_conf->cpus,
-						 gres_slurmd_conf->cpus_bitmap,
-						 gres_slurmd_conf->file,
-						 gres_slurmd_conf->type_name,
-						 gres_slurmd_conf->links,
-						 gres_slurmd_conf->unique_id,
-						 gres_slurmd_conf->config_flags);
+						 gres_slurmd_conf);
+			gres_slurmd_conf->count = count;
 			continue;
 		}
 
@@ -463,23 +460,22 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 		 * Break down record into individual devices.
 		 */
 		hl = hostlist_create(gres_slurmd_conf->file);
+		tmp_file = gres_slurmd_conf->file;
 		while ((hl_name = hostlist_shift(hl))) {
+			gres_slurmd_conf->count = 1;
+			gres_slurmd_conf->file = hl_name;
 			/*
 			 * Split this record into multiple single-GPU,
 			 * single-file records and add to single-GPU GRES list
 			 */
 			add_gres_to_list(gres_list_conf_single,
-					 gres_slurmd_conf->name, 1,
-					 gres_slurmd_conf->cpu_cnt,
-					 gres_slurmd_conf->cpus,
-					 gres_slurmd_conf->cpus_bitmap, hl_name,
-					 gres_slurmd_conf->type_name,
-					 gres_slurmd_conf->links,
-					 gres_slurmd_conf->unique_id,
-					 gres_slurmd_conf->config_flags);
+					 gres_slurmd_conf);
 			free(hl_name);
+			gres_slurmd_conf->file = NULL;
 		}
 		hostlist_destroy(hl);
+		gres_slurmd_conf->count = count;
+		gres_slurmd_conf->file = tmp_file;
 	}
 	list_iterator_destroy(itr);
 
@@ -605,7 +601,7 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 
 	/* Print out all the leftover system GPUs that are not being used */
 	if (gres_list_system && list_count(gres_list_system)) {
-		info("WARNING: The following autodetected GPUs are being ignored:");
+		warning("The following autodetected GPUs are being ignored:");
 		print_gres_list(gres_list_system, LOG_LEVEL_INFO);
 	}
 
@@ -660,15 +656,11 @@ static void _add_fake_gpus_from_file(List gres_list_system,
 		char *save_ptr = NULL;
 		char *tok;
 		int i = 0;
-		int cpu_count = 0;
-		char *cpu_range = NULL;
-		char *device_file = NULL;
-		char *type = NULL;
-		char *links = NULL;
-		char *unique_id = NULL;
-		char *flags_str = NULL;
-		uint32_t flags = 0;
-		bitstr_t *cpu_aff_mac_bitstr = NULL;
+		gres_slurmd_conf_t gres_slurmd_conf = {
+			.count = 1,
+			.name = "gpu",
+		};
+
 		line_number++;
 
 		/*
@@ -695,29 +687,31 @@ static void _add_fake_gpus_from_file(List gres_list_system,
 
 			switch (i) {
 			case 0:
-				type = xstrdup(tok);
+				gres_slurmd_conf.type_name = xstrdup(tok);
 				break;
 			case 1:
-				cpu_count = atoi(tok);
+				gres_slurmd_conf.cpu_cnt = atoi(tok);
 				break;
 			case 2:
 				if (tok[0] == '~')
 					// accommodate special tests
-					cpu_range = gpu_g_test_cpu_conv(tok);
+					gres_slurmd_conf.cpus =
+						gpu_g_test_cpu_conv(tok);
 				else
-					cpu_range = xstrdup(tok);
+					gres_slurmd_conf.cpus = xstrdup(tok);
 				break;
 			case 3:
-				links = xstrdup(tok);
+				gres_slurmd_conf.links = xstrdup(tok);
 				break;
 			case 4:
-				device_file = xstrdup(tok);
+				gres_slurmd_conf.file = xstrdup(tok);
 				break;
 			case 5:
-				unique_id = xstrdup(tok);
+				gres_slurmd_conf.unique_id = xstrdup(tok);
 				break;
 			case 6:
-				flags_str = xstrdup(tok);
+				gres_slurmd_conf.config_flags =
+					gres_flags_parse(tok, NULL, NULL);
 				break;
 			default:
 				error("Malformed line: too many data fields");
@@ -731,24 +725,21 @@ static void _add_fake_gpus_from_file(List gres_list_system,
 			error("Line #%d in fake_gpus.conf failed to parse! Make sure that the line has no empty tokens and that the format is <type>|<sys_cpu_count>|<cpu_range>|<links>|<device_file>[|<unique_id>[|<flags>]]",
 			      line_number);
 
-		cpu_aff_mac_bitstr = bit_alloc(cpu_count);
-		if (bit_unfmt(cpu_aff_mac_bitstr, cpu_range))
+		gres_slurmd_conf.cpus_bitmap =
+			bit_alloc(gres_slurmd_conf.cpu_cnt);
+		if (bit_unfmt(gres_slurmd_conf.cpus_bitmap,
+			      gres_slurmd_conf.cpus))
 			fatal("bit_unfmt() failed for CPU range: %s",
-			      cpu_range);
-
-		flags = gres_flags_parse(flags_str, NULL, NULL);
+			      gres_slurmd_conf.cpus);
 
 		// Add the GPU specified by the parsed line
-		add_gres_to_list(gres_list_system, "gpu", 1, cpu_count,
-				 cpu_range, cpu_aff_mac_bitstr, device_file,
-				 type, links, unique_id, flags);
-		FREE_NULL_BITMAP(cpu_aff_mac_bitstr);
-		xfree(cpu_range);
-		xfree(device_file);
-		xfree(type);
-		xfree(links);
-		xfree(unique_id);
-		xfree(flags_str);
+		add_gres_to_list(gres_list_system, &gres_slurmd_conf);
+		FREE_NULL_BITMAP(gres_slurmd_conf.cpus_bitmap);
+		xfree(gres_slurmd_conf.cpus);
+		xfree(gres_slurmd_conf.file);
+		xfree(gres_slurmd_conf.type_name);
+		xfree(gres_slurmd_conf.links);
+		xfree(gres_slurmd_conf.unique_id);
 	}
 	fclose(f);
 }
@@ -781,8 +772,21 @@ static List _get_system_gpu_list_fake(void)
 
 extern int init(void)
 {
+	slurmdb_tres_rec_t tres_rec;
+
 	debug("loaded");
 
+	if (!running_in_slurmstepd())
+		return SLURM_SUCCESS;
+
+	memset(&tres_rec, 0, sizeof(slurmdb_tres_rec_t));
+	tres_rec.type = "gres";
+	tres_rec.name = "gpumem";
+	tres_mem_pos = assoc_mgr_find_tres_pos(&tres_rec, false);
+	tres_rec.type = "gres";
+	tres_rec.name = "gpuutil";
+	tres_util_pos = assoc_mgr_find_tres_pos(&tres_rec, false);
+
 	return SLURM_SUCCESS;
 }
 extern int fini(void)
@@ -845,8 +849,7 @@ extern int gres_p_node_config_load(List gres_conf_list,
 		}
 	}
 
-	rc = common_node_config_load(gres_conf_list, gres_name, node_config,
-				     &gres_devices);
+	rc = gres_node_config_load(gres_conf_list, node_config, &gres_devices);
 
 	/*
 	 * See what envs the gres_slurmd_conf records want to set (if one
@@ -873,20 +876,17 @@ extern void gres_p_job_set_env(char ***job_env_ptr,
 			       uint64_t gres_cnt,
 			       gres_internal_flags_t flags)
 {
-	/*
-	 * Variables are not static like in step_*_env since we could be calling
-	 * this from the slurmd where we are dealing with a different job each
-	 * time we hit this function, so we don't want to keep track of other
-	 * unrelated job's status.  This can also get called multiple times
-	 * (different prologs and such) which would also result in bad info each
-	 * call after the first.
-	 */
-	int local_inx = 0;
-	bool already_seen = false;
+	common_gres_env_t gres_env = {
+		.bit_alloc = gres_bit_alloc,
+		.env_ptr = job_env_ptr,
+		.flags = flags,
+		.gres_cnt = gres_cnt,
+		.gres_conf_flags = node_flags,
+		.gres_devices = gres_devices,
+		.is_job = true,
+	};
 
-	gres_common_gpu_set_env(job_env_ptr, gres_bit_alloc, NULL,
-				&already_seen, &local_inx, false, true, flags,
-				node_flags, gres_devices);
+	gres_common_gpu_set_env(&gres_env);
 }
 
 /*
@@ -898,37 +898,46 @@ extern void gres_p_step_set_env(char ***step_env_ptr,
 				uint64_t gres_cnt,
 				gres_internal_flags_t flags)
 {
-	static int local_inx = 0;
-	static bool already_seen = false;
+	common_gres_env_t gres_env = {
+		.bit_alloc = gres_bit_alloc,
+		.env_ptr = step_env_ptr,
+		.flags = flags,
+		.gres_cnt = gres_cnt,
+		.gres_conf_flags = node_flags,
+		.gres_devices = gres_devices,
+	};
 
-	gres_common_gpu_set_env(step_env_ptr, gres_bit_alloc, NULL,
-				&already_seen, &local_inx, false, false, flags,
-				node_flags, gres_devices);
+	gres_common_gpu_set_env(&gres_env);
 }
 
 /*
  * Reset environment variables as appropriate for a job (i.e. this one task)
  * based upon the job step's GRES state and assigned CPUs.
  */
-extern void gres_p_task_set_env(char ***step_env_ptr,
+extern void gres_p_task_set_env(char ***task_env_ptr,
 				bitstr_t *gres_bit_alloc,
 				uint64_t gres_cnt,
 				bitstr_t *usable_gres,
 				gres_internal_flags_t flags)
 {
-	static int local_inx = 0;
-	static bool already_seen = false;
+	common_gres_env_t gres_env = {
+		.bit_alloc = gres_bit_alloc,
+		.env_ptr = task_env_ptr,
+		.flags = flags,
+		.gres_cnt = gres_cnt,
+		.gres_conf_flags = node_flags,
+		.gres_devices = gres_devices,
+		.is_task = true,
+		.usable_gres = usable_gres,
+	};
 
-	gres_common_gpu_set_env(
-		step_env_ptr, gres_bit_alloc, usable_gres,
-		&already_seen, &local_inx, true, false, flags,
-		node_flags, gres_devices);
+	gres_common_gpu_set_env(&gres_env);
 }
 
 /* Send GPU-specific GRES information to slurmstepd via a buffer */
 extern void gres_p_send_stepd(buf_t *buffer)
 {
-	common_send_stepd(buffer, gres_devices);
+	gres_send_stepd(buffer, gres_devices);
 
 	pack32(node_flags, buffer);
 }
@@ -936,7 +945,7 @@ extern void gres_p_send_stepd(buf_t *buffer)
 /* Receive GPU-specific GRES information from slurmd via a buffer */
 extern void gres_p_recv_stepd(buf_t *buffer)
 {
-	common_recv_stepd(buffer, &gres_devices);
+	gres_recv_stepd(buffer, &gres_devices);
 
 	safe_unpack32(&node_flags, buffer);
 
@@ -994,34 +1003,34 @@ extern List gres_p_get_devices(void)
  * Build record used to set environment variables as appropriate for a job's
  * prolog or epilog based GRES allocated to the job.
  */
-extern gres_epilog_info_t *gres_p_epilog_build_env(
+extern gres_prep_t *gres_p_prep_build_env(
 	gres_job_state_t *gres_js)
 {
 	int i;
-	gres_epilog_info_t *gres_ei;
+	gres_prep_t *gres_prep;
 
-	gres_ei = xmalloc(sizeof(gres_epilog_info_t));
-	gres_ei->node_cnt = gres_js->node_cnt;
-	gres_ei->gres_bit_alloc = xcalloc(gres_ei->node_cnt,
+	gres_prep = xmalloc(sizeof(gres_prep_t));
+	gres_prep->node_cnt = gres_js->node_cnt;
+	gres_prep->gres_bit_alloc = xcalloc(gres_prep->node_cnt,
 					  sizeof(bitstr_t *));
-	for (i = 0; i < gres_ei->node_cnt; i++) {
+	for (i = 0; i < gres_prep->node_cnt; i++) {
 		if (gres_js->gres_bit_alloc &&
 		    gres_js->gres_bit_alloc[i]) {
-			gres_ei->gres_bit_alloc[i] =
+			gres_prep->gres_bit_alloc[i] =
 				bit_copy(gres_js->gres_bit_alloc[i]);
 		}
 	}
 
-	return gres_ei;
+	return gres_prep;
 }
 
 /*
  * Set environment variables as appropriate for a job's prolog or epilog based
  * GRES allocated to the job.
  */
-extern void gres_p_epilog_set_env(char ***epilog_env_ptr,
-				  gres_epilog_info_t *gres_ei, int node_inx)
+extern void gres_p_prep_set_env(char ***prep_env_ptr,
+				gres_prep_t *gres_prep, int node_inx)
 {
-	(void) gres_common_epilog_set_env(epilog_env_ptr, gres_ei,
+	(void) gres_common_prep_set_env(prep_env_ptr, gres_prep,
 					  node_inx, node_flags, gres_devices);
 }
