@@ -39,7 +39,6 @@
 \*****************************************************************************/
 
 #define _GNU_SOURCE
-#define _XOPEN_SOURCE 500 /* For ftw.h */
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,7 +46,6 @@
 #include <sys/mman.h>
 #include <sched.h>
 #include <fcntl.h>
-#include <ftw.h>
 #include <sys/mount.h>
 #include <linux/limits.h>
 #include <semaphore.h>
@@ -82,7 +80,6 @@ const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
 
 static slurm_jc_conf_t *jc_conf = NULL;
 static int step_ns_fd = -1;
-static bool force_rm = true;
 
 static void _create_paths(uint32_t job_id, char **job_mount, char **ns_holder,
 			  char **src_bind)
@@ -293,9 +290,9 @@ static int _mount_private_dirs(char *path, uid_t uid)
 			      __func__, mount_path);
 			goto private_mounts_exit;
 		}
-		rc = chown(mount_path, uid, -1);
+		rc = lchown(mount_path, uid, -1);
 		if (rc) {
-			error("%s: chown failed for %s: %m",
+			error("%s: lchown failed for %s: %m",
 			      __func__, mount_path);
 			goto private_mounts_exit;
 		}
@@ -348,43 +345,6 @@ static int _mount_private_shm(void)
 	return rc;
 }
 
-static int _rm_data(const char *path, const struct stat *st_buf,
-		    int type, struct FTW *ftwbuf)
-{
-	int rc = SLURM_SUCCESS;
-
-	/*
-	 * ftwbuf->level == 0 means path is the initial path passed to nftw
-	 * We expect this rmdir to fail since it is a mount point.  Just skip it
-	 * and expect that it will be removed later.
-	 */
-	if (ftwbuf->level == 0)
-		return SLURM_SUCCESS;
-
-	if (remove(path) < 0) {
-		log_level_t log_lvl;
-		if (force_rm) {
-			rc = SLURM_ERROR;
-			log_lvl = LOG_LEVEL_ERROR;
-		} else
-			log_lvl = LOG_LEVEL_DEBUG2;
-
-		if (type == FTW_NS)
-			log_var(log_lvl,
-					"%s: Unreachable file of FTW_NS type: %s",
-					__func__, path);
-		else if (type == FTW_DNR)
-			log_var(log_lvl,
-					"%s: Unreadable directory: %s",
-					__func__, path);
-
-		log_var(log_lvl, "%s: could not remove path: %s: %m",
-			__func__, path);
-	}
-
-	return rc;
-}
-
 static int _clean_job_basepath(uint32_t job_id)
 {
 	DIR *dp;
@@ -432,18 +392,10 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 
 	_create_paths(job_id, &job_mount, &ns_holder, &src_bind);
 
-	rc = mkdir(job_mount, 0700);
-	if (rc && errno != EEXIST) {
+	if (mkdir(job_mount, 0700)) {
 		error("%s: mkdir %s failed: %m", __func__, job_mount);
 		rc = SLURM_ERROR;
 		goto end_it;
-	} else if (rc && errno == EEXIST) {
-		/*
-		 * This is coming from sbcast likely,
-		 * exit as success
-		 */
-		rc = 0;
-		goto exit2;
 	}
 
 #if !defined(__APPLE__) && !defined(__FreeBSD__)
@@ -717,11 +669,11 @@ exit1:
 
 exit2:
 	if (rc) {
+		int failures;
 		/* cleanup the job mount */
-		force_rm = true;
-		if (nftw(job_mount, _rm_data, 64, FTW_DEPTH|FTW_PHYS) < 0) {
-			error("%s: Directory traversal failed: %s: %m",
-			      __func__, job_mount);
+		if ((failures = rmdir_recursive(job_mount, false))) {
+			error("%s: failed to remove %d files from %s",
+			      __func__, failures, job_mount);
 			rc = SLURM_ERROR;
 			goto end_it;
 		}
@@ -815,7 +767,7 @@ extern int container_p_join(uint32_t job_id, uid_t uid)
 static int _delete_ns(uint32_t job_id)
 {
 	char *job_mount = NULL, *ns_holder = NULL;
-	int rc = 0;
+	int rc = 0, failures = 0;
 
 #ifdef HAVE_NATIVE_CRAY
 	return SLURM_SUCCESS;
@@ -857,25 +809,9 @@ static int _delete_ns(uint32_t job_id)
 		}
 	}
 
-	/*
-	 * Traverses the job directory, and delete all files.
-	 * Doesn't -
-	 *	traverse filesystem boundaries,
-	 *	follow symbolic links
-	 * Does -
-	 *	a post order traversal and delete directory after processing
-	 *      contents
-	 * NOTE: Can happen EBUSY here so we need to ignore this.
-	 */
-	force_rm = false;
-	if (nftw(job_mount, _rm_data, 64, FTW_DEPTH|FTW_PHYS) < 0) {
-		error("%s: Directory traversal failed: %s: %m",
-		      __func__, job_mount);
-		xfree(job_mount);
-		xfree(ns_holder);
-		return SLURM_ERROR;
-	}
-
+	if ((failures = rmdir_recursive(job_mount, false)))
+		error("%s: failed to remove %d files from %s",
+		      __func__, failures, job_mount);
 	if (umount2(job_mount, MNT_DETACH))
 		debug2("umount2: %s failed: %m", job_mount);
 	rmdir(job_mount);
