@@ -43,41 +43,57 @@
 #include "switch_hpe_slingshot.h"
 
 #include <dlfcn.h>
+#include <sys/stat.h>
 
 #include "libcxi/libcxi.h"
 
 /* Global variables */
 static void *cxi_handle = NULL;
 static bool cxi_avail = false;
-static struct cxil_dev **cxi_devs;
+static struct cxil_dev **cxi_devs = NULL;
 static int cxi_ndevs = 0;
 
+/* Define struct not defined in earlier versions of libcxi */
+#ifndef HAVE_STRUCT_CXI_RSRC_USE
+#define CXI_RSRC_TYPE_MAX 8
+struct cxi_rsrc_use {
+	uint16_t in_use[CXI_RSRC_TYPE_MAX];
+};
+#endif
+
 /* Function pointers loaded from libcxi */
 static int (*cxil_get_device_list_p)(struct cxil_device_list **);
+static int (*cxil_get_svc_list_p)(struct cxil_dev *dev,
+				  struct cxil_svc_list **svc_list);
 static int (*cxil_open_device_p)(uint32_t, struct cxil_dev **);
 static int (*cxil_alloc_svc_p)(struct cxil_dev *, struct cxi_svc_desc *,
 	struct cxi_svc_fail_info *);
 static int (*cxil_destroy_svc_p)(struct cxil_dev *, unsigned int);
+#if HAVE_STRUCT_CXI_RSRC_USE
+static int (*cxil_get_svc_rsrc_use_p)(struct cxil_dev *, unsigned int,
+				      struct cxi_rsrc_use *);
+#endif
 
 
-#define LOOKUP_SYM(_lib, _version, x) \
+#define LOOKUP_SYM(_lib, x) \
 do { \
-	x ## _p = (_version && _version[0]) ? \
-		dlvsym(_lib, #x, _version) : dlsym(_lib, #x); \
+	x ## _p = dlsym(_lib, #x); \
 	if (x ## _p == NULL) { \
-		error("Error loading symbol %s, version '%s': %s", \
-			#x, _version, dlerror()); \
-		return false; \
+		log_flag(SWITCH, "Error loading symbol: %s (skipped)", \
+			 dlerror()); \
 	} \
 } while (0)
 
-static bool _load_cxi_funcs(void *lib, char *version)
+static bool _load_cxi_funcs(void *lib)
 {
-	LOOKUP_SYM(lib, version, cxil_get_device_list);
-	LOOKUP_SYM(lib, version, cxil_open_device);
-	LOOKUP_SYM(lib, version, cxil_alloc_svc);
-	LOOKUP_SYM(lib, version, cxil_destroy_svc);
-
+	LOOKUP_SYM(lib, cxil_get_device_list);
+	LOOKUP_SYM(lib, cxil_get_svc_list);
+	LOOKUP_SYM(lib, cxil_open_device);
+	LOOKUP_SYM(lib, cxil_alloc_svc);
+	LOOKUP_SYM(lib, cxil_destroy_svc);
+#if HAVE_STRUCT_CXI_RSRC_USE
+	LOOKUP_SYM(lib, cxil_get_svc_rsrc_use);
+#endif
 	return true;
 }
 
@@ -112,39 +128,80 @@ static void _print_devinfo(int dev, struct cxil_devinfo *info)
 }
 
 /*
- * Return array of limits already reserved by system services
+ * Subtract MAX(reserved limit, current in_use value) for this system service
+ * from the max value for this limit specified in the device info;
+ * this will be used as the max value for the device that users can request
  */
-static bool _get_reserved_limits(int dev, slingshot_limits_set_t *limits)
+static void _adjust_limit(int rsrc, int dev, int svc, struct cxi_limits *lim,
+			  uint16_t *in_use, uint16_t *devmax)
+{
+	uint16_t oldmax = *devmax;
+	uint16_t adjust = MAX(lim[rsrc].res, in_use[rsrc]);
+	*devmax -= adjust;
+	log_flag(SWITCH, "CXI dev/svc[%d][%d]: limits.%s.res/in_use %hu %hu (old/new max %hu %hu)",
+		 dev, svc, cxi_rsrc_type_strs[rsrc], lim[rsrc].res,
+		 in_use[rsrc], oldmax, *devmax);
+}
+
+/*
+ * For this device, total the reserved resources used by system services,
+ * and remove that amount from the device's max limits (this will be used
+ * as a ceiling for resource requests)
+ */
+static bool _adjust_dev_limits(int dev, struct cxil_devinfo *devinfo)
 {
 	int svc, rc;
+	int num_svc = 0, num_system_svc = 0;
 	struct cxil_svc_list *list = NULL;
 
 	if (!cxi_devs[dev])
 		return true;
-	if ((rc = cxil_get_svc_list(cxi_devs[dev], &list))) {
-		error("Could not get service list for CXI device[%d] dev_id=%d (%s): %d",
-			dev, cxi_devs[dev]->info.dev_id,
-			cxi_devs[dev]->info.device_name, rc);
+	if ((rc = cxil_get_svc_list_p(cxi_devs[dev], &list))) {
+		error("Could not get service list for CXI dev_id=%d (%s): %s",
+		      devinfo->dev_id, devinfo->device_name, strerror(-rc));
 		return false;
 	}
 	for (svc = 0; svc < list->count; svc++) {
-#define PLIMIT(DEV, SVC, LIM) { \
-	limits->LIM.res += list->descs[SVC].limits.LIM.res; \
-	log_flag(SWITCH, "CXI dev/svc/system[%d][%d][%d]: limits.%s.res %hu (tot/max %hu %hu)", \
-		 DEV, SVC, list->descs[SVC].is_system_svc, #LIM, \
-		 list->descs[SVC].limits.LIM.res, limits->LIM.res, \
-		 list->descs[SVC].limits.LIM.max); \
-}
-		PLIMIT(dev, svc, ptes);
-		PLIMIT(dev, svc, txqs);
-		PLIMIT(dev, svc, tgqs);
-		PLIMIT(dev, svc, eqs);
-		PLIMIT(dev, svc, cts);
-		PLIMIT(dev, svc, acs);
-		PLIMIT(dev, svc, tles);
-		PLIMIT(dev, svc, les);
-#undef PLIMIT
+		struct cxi_rsrc_use usage = { 0 };
+		struct cxi_limits *lim;
+
+		if (!list->descs[svc].is_system_svc) {
+			num_svc++;
+			continue;
+		}
+		num_system_svc++;
+#if HAVE_STRUCT_CXI_RSRC_USE
+		if (cxil_get_svc_rsrc_use_p &&
+		    (rc = cxil_get_svc_rsrc_use_p(cxi_devs[dev],
+						  list->descs[svc].svc_id,
+						  &usage))) {
+			error("Could not get resource usage for CXI dev_id=%d (%s) svc_id=%d: %s",
+			      devinfo->dev_id, devinfo->device_name,
+			      list->descs[svc].svc_id, strerror(-rc));
+			// in_use value will be 0
+		}
+#endif
+		lim = list->descs[svc].limits.type;
+		_adjust_limit(CXI_RSRC_TYPE_PTE, dev, svc, lim,
+			      usage.in_use, &devinfo->num_ptes);
+		_adjust_limit(CXI_RSRC_TYPE_TXQ, dev, svc, lim,
+			      usage.in_use, &devinfo->num_txqs);
+		_adjust_limit(CXI_RSRC_TYPE_TGQ, dev, svc, lim,
+			      usage.in_use, &devinfo->num_tgqs);
+		_adjust_limit(CXI_RSRC_TYPE_EQ, dev, svc, lim,
+			      usage.in_use, &devinfo->num_eqs);
+		_adjust_limit(CXI_RSRC_TYPE_CT, dev, svc, lim,
+			      usage.in_use, &devinfo->num_cts);
+		_adjust_limit(CXI_RSRC_TYPE_LE, dev, svc, lim,
+			      usage.in_use, &devinfo->num_les);
+		_adjust_limit(CXI_RSRC_TYPE_TLE, dev, svc, lim,
+			      usage.in_use, &devinfo->num_tles);
+		_adjust_limit(CXI_RSRC_TYPE_AC, dev, svc, lim,
+			      usage.in_use, &devinfo->num_acs);
 	}
+	log_flag(SWITCH, "CXI services=%d system=%d user=%d",
+		 list->count, num_system_svc, num_svc);
+
 	free(list);	/* can't use xfree() */
 	return true;
 }
@@ -152,14 +209,14 @@ static bool _get_reserved_limits(int dev, slingshot_limits_set_t *limits)
 /*
  * Set up basic access to the CXI devices in the daemon
  */
-static bool _create_cxi_devs(void)
+static bool _create_cxi_devs(slingshot_jobinfo_t *job)
 {
 	struct cxil_device_list *list;
 	int dev, rc;
 
 	if ((rc = cxil_get_device_list_p(&list))) {
-		error("Could not get a list of the CXI devices: %d %d",
-		      rc, errno);
+		error("Could not get a list of the CXI devices: %s",
+		      strerror(-rc));
 		return false;
 	}
 
@@ -173,20 +230,23 @@ static bool _create_cxi_devs(void)
 	cxi_ndevs = list->count;
 
 	/* We're OK with only getting access to a subset */
-	slingshot_limits_set_t reslimits = { 0 };
 	for (dev = 0; dev < cxi_ndevs; dev++) {
 		struct cxil_devinfo *info = &list->info[dev];
 		if ((rc = cxil_open_device_p(info->dev_id, &cxi_devs[dev]))) {
-			error("Could not open CXI device[%d] dev_id=%d (%s): %d",
-			      dev, info->dev_id, info->device_name, rc);
+			error("Could not open CXI device dev_id=%d (%s): %s",
+			      info->dev_id, info->device_name, strerror(-rc));
 			cxi_devs[dev] = NULL;
 			continue;
 		}
 		/* Only done in debug mode */
 		if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH)
 			_print_devinfo(dev, &cxi_devs[dev]->info);
-		if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH)
-			_get_reserved_limits(dev, &reslimits);
+		/*
+		 * If configured, adjust max NIC resources available
+		 * by subtracting system service reserved/used values
+		 */
+		if (job->flags & SLINGSHOT_FLAGS_ADJUST_LIMITS)
+			_adjust_dev_limits(dev, &cxi_devs[dev]->info);
 	}
 
 	return true;
@@ -196,9 +256,9 @@ static bool _create_cxi_devs(void)
  * Return a cxi_limits struct with res/max fields set according to
  * job max/res/def limits, device max limits, and number of CPUs on node
  */
-static struct cxi_limits set_desc_limits(const char *name,
-					 const slingshot_limits_t *joblimits,
-					 uint16_t dev_max, int ncpus)
+static struct cxi_limits _set_desc_limits(int rsrc,
+					  const slingshot_limits_t *joblimits,
+					  uint16_t dev_max, int ncpus)
 {
 	struct cxi_limits ret;
 
@@ -208,6 +268,12 @@ static struct cxi_limits set_desc_limits(const char *name,
 	ret.res = joblimits->res ? joblimits->res : (joblimits->def * ncpus);
 	/* Reserved can't be higher than max */
 	ret.res = MIN(ret.res, ret.max);
+	/*
+	 * SPECIAL CASE: limit TLE max value to reserved value
+	 */
+	if (rsrc == CXI_RSRC_TYPE_TLE)
+		ret.max = ret.res;
+	const char *name = cxi_rsrc_type_strs[rsrc];
 	log_flag(SWITCH, "job %s.max/res/def/cpus %hu %hu %hu %d"
 		 " CXI desc %s.max/res %hu %hu",
 		 name, joblimits->max, joblimits->res, joblimits->def, ncpus,
@@ -227,14 +293,10 @@ static void _create_cxi_descriptor(struct cxi_svc_desc *desc,
 
 	memset(desc, 0, sizeof(*desc));
 
-#ifdef CXI_SVC_MEMBER_UID
 	desc->restricted_members = true;
 	desc->members[0].type = CXI_SVC_MEMBER_UID;
 	desc->members[0].svc_member.uid = uid;
 	desc->members[1].type = CXI_SVC_MEMBER_IGNORE;
-#else
-	desc->restricted_members = false;
-#endif
 
 	/* Set up VNI */
 	if (job->num_vnis > 0) {
@@ -273,22 +335,22 @@ static void _create_cxi_descriptor(struct cxi_svc_desc *desc,
 	 * otherwise use the number of CPUs for this step
 	 */
 	cpus = job->depth ? job->depth : step_cpus;
-	desc->limits.txqs = set_desc_limits("txqs", &job->limits.txqs,
-					    devinfo->num_txqs, cpus);
-	desc->limits.tgqs = set_desc_limits("tgqs", &job->limits.tgqs,
-					    devinfo->num_tgqs, cpus);
-	desc->limits.eqs = set_desc_limits("eqs", &job->limits.eqs,
-					    devinfo->num_eqs, cpus);
-	desc->limits.cts = set_desc_limits("cts", &job->limits.cts,
-					    devinfo->num_cts, cpus);
-	desc->limits.tles = set_desc_limits("tles", &job->limits.tles,
-					    devinfo->num_tles, cpus);
-	desc->limits.ptes = set_desc_limits("ptes", &job->limits.ptes,
-					    devinfo->num_ptes, cpus);
-	desc->limits.les = set_desc_limits("les", &job->limits.les,
-					    devinfo->num_les, cpus);
-	desc->limits.acs = set_desc_limits("acs", &job->limits.acs,
-					    devinfo->num_acs, cpus);
+	desc->limits.txqs = _set_desc_limits(CXI_RSRC_TYPE_TXQ,
+				&job->limits.txqs, devinfo->num_txqs, cpus);
+	desc->limits.tgqs = _set_desc_limits(CXI_RSRC_TYPE_TGQ,
+				&job->limits.tgqs, devinfo->num_tgqs, cpus);
+	desc->limits.eqs = _set_desc_limits(CXI_RSRC_TYPE_EQ,
+				&job->limits.eqs, devinfo->num_eqs, cpus);
+	desc->limits.cts = _set_desc_limits(CXI_RSRC_TYPE_CT,
+				&job->limits.cts, devinfo->num_cts, cpus);
+	desc->limits.tles = _set_desc_limits(CXI_RSRC_TYPE_TLE,
+				&job->limits.tles, devinfo->num_tles, cpus);
+	desc->limits.ptes = _set_desc_limits(CXI_RSRC_TYPE_PTE,
+				&job->limits.ptes, devinfo->num_ptes, cpus);
+	desc->limits.les = _set_desc_limits(CXI_RSRC_TYPE_LE,
+				&job->limits.les, devinfo->num_les, cpus);
+	desc->limits.acs = _set_desc_limits(CXI_RSRC_TYPE_AC,
+				&job->limits.acs, devinfo->num_acs, cpus);
 
 	/* Differentiates system and user services */
 	desc->is_system_svc = false;
@@ -298,38 +360,25 @@ static void _create_cxi_descriptor(struct cxi_svc_desc *desc,
  * Open the Slingshot CXI library; set up functions and set cxi_avail
  * if successful (default is 'false')
  */
-extern bool slingshot_open_cxi_lib(void)
+extern bool slingshot_open_cxi_lib(slingshot_jobinfo_t *job)
 {
-	char *libfile;
-	char *version;
-
-	if (!(libfile = getenv(SLINGSHOT_CXI_LIB_ENV)))
-		libfile = SLINGSHOT_CXI_LIB;
-
-	if (!libfile || (libfile[0] == '\0')) {
-		error("Bad library file specified by %s variable",
-		      SLINGSHOT_CXI_LIB_ENV);
+	if (!(cxi_handle = dlopen(HPE_SLINGSHOT_LIB,
+				  RTLD_LAZY | RTLD_GLOBAL))) {
+		error("Couldn't find CXI library %s: %s",
+		      HPE_SLINGSHOT_LIB, dlerror());
 		goto out;
 	}
 
-	if (!(cxi_handle = dlopen(libfile, RTLD_LAZY | RTLD_GLOBAL))) {
-		error("Couldn't find CXI library %s: %s", libfile, dlerror());
-		goto out;
-	}
-
-	if (!(version = getenv(SLINGSHOT_CXI_LIB_VERSION_ENV)))
-		version = SLINGSHOT_CXI_LIB_VERSION;
-
-	debug("CXI library %s, version '%s'", libfile, version);
-	if (!_load_cxi_funcs(cxi_handle, version))
+	if (!_load_cxi_funcs(cxi_handle))
 		goto out;
 
-	if (!_create_cxi_devs())
+	if (!_create_cxi_devs(job))
 		goto out;
 
 	cxi_avail = true;
 out:
 	log_flag(SWITCH, "cxi_avail=%d", cxi_avail);
+
 	return cxi_avail;
 }
 
@@ -345,15 +394,42 @@ static struct cxil_dev *_device_name_to_dev(const char *devname)
 		if (!xstrcmp(devname, cxi_devs[dev]->info.device_name))
 			return cxi_devs[dev];
 	}
+
 	return NULL;
 }
 
+/*
+ * Attempt to destroy a CXI service; retry a few times on EBUSY
+ */
+static bool _destroy_cxi_service(struct cxil_dev *dev, const char *devname,
+				 int svc_id)
+{
+	int i, rc;
+
+	for (i = 0; i < SLINGSHOT_CXI_DESTROY_RETRIES; i++) {
+		debug("Destroying CXI SVC ID %d on NIC %s (retry %d)",
+		      svc_id, devname, i);
+		rc = cxil_destroy_svc_p(dev, svc_id);
+		if (rc == 0)
+			return true;
+		error("Failed to destroy CXI Service ID %d (%s): %s",
+		      svc_id, devname, strerror(-rc));
+		if (rc != -EBUSY)
+			break;
+		sleep(1);
+	}
+	return false;
+}
+
 /*
  * In the daemon, when the shepherd for an App terminates, free any CXI
  * Services we have allocated for it
  */
-extern bool slingshot_destroy_services(slingshot_jobinfo_t *job)
+extern bool slingshot_destroy_services(slingshot_jobinfo_t *job,
+				       uint32_t job_id)
 {
+	bool retval = true;
+
 	xassert(job);
 
 	if (!cxi_avail)
@@ -375,109 +451,86 @@ extern bool slingshot_destroy_services(slingshot_jobinfo_t *job)
 			continue;
 		}
 
-		debug("Destroying CXI SVC ID %d on NIC %s",
-			svc_id, devname);
-
-		int rc = cxil_destroy_svc_p(dev, svc_id);
-		if (rc) {
-			error("Failed to destroy CXI Service ID %d (%s): %d",
-			      svc_id, devname, rc);
-			return false;
-		}
+		/* Try to destroy service (with retries) */
+		if (!_destroy_cxi_service(dev, devname, svc_id))
+			retval = false;
 	}
 
 	xfree(job->profiles);
 	job->profiles = NULL;
 	job->num_profiles = 0;
-	return true;
+
+	return retval;
+}
+
+/*
+ * Log any non-system CXI services, to help with diagnosing allocation failures
+ */
+static void _log_other_services(struct cxil_dev *dev)
+{
+	struct cxil_svc_list *list = NULL;
+	int rc;
+
+	if ((rc = cxil_get_svc_list(dev, &list))) {
+		error("Could not get service list for CXI dev_id=%d (%s): %s",
+		      dev->info.dev_id, dev->info.device_name, strerror(-rc));
+		return;
+	}
+	for (int svc = 0; svc < list->count; svc++) {
+		if (list->descs[svc].is_system_svc)
+			continue;
+		error("CXI allocation failed for %s: svc_id=%d UID=%d also on device",
+		      dev->info.device_name, list->descs[svc].svc_id,
+		      list->descs[svc].members[0].svc_member.uid);
+	}
+	free(list);	/* can't use xfree() */
 }
 
 /*
  * If cxil_alloc_svc failed, log information about the failure
  */
-static void _alloc_fail_info(const struct cxil_dev *dev,
-			     const struct cxi_svc_desc *desc,
-			     const struct cxi_svc_fail_info *fail_info)
+static void _alloc_fail_info(struct cxil_dev *dev,
+			     struct cxi_svc_desc *desc,
+			     struct cxi_svc_fail_info *fail_info)
 {
-	error("Slingshot service allocation failed on %s",
-	      dev->info.device_name);
-
 	for (int rsrc = 0; rsrc < CXI_RSRC_TYPE_MAX; rsrc++) {
-		char *rsrc_str = NULL;
-		int rsrc_req = 0;
+		const char *rsrc_str = cxi_rsrc_type_strs[rsrc];
+		int rsrc_res = desc->limits.type[rsrc].res;
+		int rsrc_max = desc->limits.type[rsrc].max;
 
-		switch (rsrc) {
-		case CXI_RSRC_TYPE_PTE:
-			rsrc_str = "portal table entries";
-			rsrc_req = desc->limits.ptes.res;
-			break;
-		case CXI_RSRC_TYPE_TXQ:
-			rsrc_str = "transmit command queues";
-			rsrc_req = desc->limits.txqs.res;
-			break;
-		case CXI_RSRC_TYPE_TGQ:
-			rsrc_str = "target command queues";
-			rsrc_req = desc->limits.tgqs.res;
-			break;
-		case CXI_RSRC_TYPE_EQ:
-			rsrc_str = "event queues";
-			rsrc_req = desc->limits.eqs.res;
-			break;
-		case CXI_RSRC_TYPE_CT:
-			rsrc_str = "counters";
-			rsrc_req = desc->limits.cts.res;
-			break;
-		case CXI_RSRC_TYPE_LE:
-			rsrc_str = "list entries";
-			rsrc_req = desc->limits.les.res;
-			break;
-		case CXI_RSRC_TYPE_TLE:
-			rsrc_str = "trigger list entries";
-			rsrc_req = desc->limits.tles.res;
-			break;
-		case CXI_RSRC_TYPE_AC:
-			rsrc_str = "addressing contexts";
-			rsrc_req = desc->limits.acs.res;
-			break;
-		case CXI_RSRC_TYPE_MAX:
-		default:
-			rsrc_str = "invalid resource";
-			break;
-		}
-
-		if (rsrc_req > fail_info->rsrc_avail[rsrc])
-			error("Only %d %s available on %s (requested %d)",
-			      fail_info->rsrc_avail[rsrc], rsrc_str,
-			      dev->info.device_name, rsrc_req);
+		if (rsrc_res > fail_info->rsrc_avail[rsrc])
+			error("%s: allocation failed: max/available/requested %d %d %d",
+			      rsrc_str, rsrc_max, fail_info->rsrc_avail[rsrc],
+			      rsrc_res);
 	}
 
 	if (fail_info->no_le_pools)
-		error("No list entry pools available on %s",
-		      dev->info.device_name);
+		error("No LE pools available on %s", dev->info.device_name);
 	if (fail_info->no_tle_pools)
-		error("No trigger list entry pools available on %s",
-		      dev->info.device_name);
+		error("No TLE pools available on %s", dev->info.device_name);
 	if (fail_info->no_cntr_pools)
-		error("No counter pools available on %s",
-		      dev->info.device_name);
+		error("No CNTR pools available on %s", dev->info.device_name);
+
+	/* log any other non-system services on this node */
+	_log_other_services(dev);
 }
 
 /*
  * Set up CXI services for each of the CXI NICs on this host
  */
 extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
-				      uint16_t step_cpus)
+				      uint16_t step_cpus, uint32_t job_id)
 {
 	int prof, devn;
 	struct cxi_svc_desc desc;
 	struct cxil_dev *dev;
 	struct cxi_svc_fail_info failinfo;
-	pals_comm_profile_t *profile;
+	slingshot_comm_profile_t *profile;
 
 	xassert(job);
 
 	/* dlopen() libcxi and query CXI devices */
-	slingshot_open_cxi_lib();
+	slingshot_open_cxi_lib(job);
 
 	/* Just return true if CXI not available or no VNIs to set up */
 	if (!cxi_avail || !job->num_vnis) {
@@ -506,6 +559,8 @@ extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
 
 		int svc_id = cxil_alloc_svc_p(dev, &desc, &failinfo);
 		if (svc_id < 0) {
+			error("Slingshot service allocation failed on %s: %s",
+			      dev->info.device_name, strerror(-svc_id));
 			_alloc_fail_info(dev, &desc, &failinfo);
 			goto error;
 		}
@@ -520,16 +575,18 @@ extern bool slingshot_create_services(slingshot_jobinfo_t *job, uint32_t uid,
 			"%s", dev->info.device_name);
 
 		debug("Creating CXI profile[%d] on NIC %d (%s):"
-			" SVC ID %u vnis=[%hu %hu %hu %hu] tcs=%u",
+			" SVC ID %u vnis=[%hu %hu %hu %hu] tcs=%#x",
 			prof, devn, profile->device_name, profile->svc_id,
 			profile->vnis[0], profile->vnis[1], profile->vnis[2],
 			profile->vnis[3], profile->tcs);
 		prof++;
 	}
+
 	return true;
 
 error:
-	slingshot_destroy_services(job);
+	slingshot_destroy_services(job, job_id);
+
 	return false;
 }
 
