@@ -107,7 +107,7 @@ main (int argc, char **argv)
 	log_options_t log_opts = LOG_OPTS_STDERR_ONLY ;
 	int rc = 0;
 
-	slurm_conf_init(NULL);
+	slurm_init(NULL);
 	log_init (xbasename(argv[0]), log_opts, SYSLOG_FACILITY_DAEMON, NULL);
 	initialize_and_process_args(argc, argv);
 	if (opt.verbose) {
@@ -178,12 +178,16 @@ static void
 _load_job_records (void)
 {
 	int error_code;
+	uint16_t show_flags = 0;
+
+	show_flags |= SHOW_ALL;
+	show_flags |= opt.clusters ? SHOW_LOCAL : SHOW_FEDERATION;
 
 	/* We need the fill job array string representation for identifying
 	 * and killing job arrays */
 	setenv("SLURM_BITSTR_LEN", "0", 1);
-	error_code = slurm_load_jobs ((time_t) NULL, &job_buffer_ptr,
-				      SHOW_ALL | SHOW_FEDERATION);
+	error_code = slurm_load_jobs((time_t) NULL, &job_buffer_ptr,
+				     show_flags);
 
 	if (error_code) {
 		slurm_perror ("slurm_load_jobs error");
@@ -729,14 +733,24 @@ _cancel_job_id (void *ci)
 		flags |= KILL_JOB_BATCH;
 		job_type = "batch ";
 	}
+
+	/*
+	 * With the introduction of the ScronParameters=explicit_scancel option,
+	 * scancel requests for a cron job should be rejected unless the --cron
+	 * flag is specified.
+	 * To prevent introducing this option from influencing anything other
+	 * than user requests, it has been set up so that when KILL_NO_CRON is
+	 * set when explicit_scancel is also set, the request will be rejected.
+	 */
+	if (!opt.cron)
+		flags |= KILL_NO_CRON;
+
 	if (opt.full) {
 		flags |= KILL_FULL_JOB;
 		job_type = "full ";
 	}
 	if (opt.hurry)
 		flags |= KILL_HURRY;
-	if (cancel_info->array_flag)
-		flags |= KILL_JOB_ARRAY;
 
 	if (!cancel_info->job_id_str) {
 		if (cancel_info->array_job_id &&
