@@ -2,7 +2,7 @@
  *  gres_ctld.h - Functions for gres used only in the slurmctld
  *****************************************************************************
  *  Copyright (C) 2020 SchedMD LLC.
- *  Derived in large part from code previously in common/gres.h
+ *  Derived in large part from code previously in interfaces/gres.h
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -37,7 +37,7 @@
 #ifndef _GRES_CTLD_H
 #define _GRES_CTLD_H
 
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 
 /*
  * Fill in job_gres_list with the total amount of GRES on a node.
@@ -136,8 +136,12 @@ extern void gres_ctld_job_merge(List from_job_gres_list,
 				List to_job_gres_list,
 				bitstr_t *to_job_node_bitmap);
 
-/* Clear any vestigial job gres state. This may be needed on job requeue. */
-extern void gres_ctld_job_clear(List job_gres_list);
+/*
+ * Clear any vestigial alloc job gres state. This may be needed on job requeue.
+ * This only clears out the allocated portions of the gres list, it does not
+ * remove the actual items from the list.
+ */
+extern void gres_ctld_job_clear_alloc(List job_gres_list);
 
 /* Given a job's GRES data structure, return the indecies for selected elements
  * IN job_gres_list  - job's allocated GRES data structure
@@ -174,6 +178,47 @@ extern void gres_ctld_set_node_tres_cnt(List gres_list,
 					uint64_t *tres_cnt,
 					bool locked);
 
+/*
+ * Determine how many cores of a job's allocation can be allocated to a step
+ *	on a specific node
+ * IN job_gres_list - a running job's allocated gres info
+ * IN/OUT step_gres_list - a pending job step's gres requirements
+ * IN node_offset - index into the job's node allocation
+ * IN first_step_node - true if this is node zero of the step (do initing)
+ * IN cpus_per_task - number of CPUs required per task
+ * IN max_rem_nodes - maximum nodes remaining for step (including this one)
+ * IN ignore_alloc - if set ignore resources already allocated to running steps
+ * IN job_id, step_id - ID of the step being allocated.
+ * IN test_mem - true if we should test if mem_per_gres would exceed a limit.
+ * IN job_resrcs_ptr - pointer to this job's job_resources_t; used to know
+ *                     how much of the job's memory is available.
+ * OUT err_code - If an error occurred, set this to tell the caller why the
+ *                error happend.
+ * RET Count of available cores on this node (sort of):
+ *     NO_VAL64 if no limit or 0 if node is not usable
+ */
+extern uint64_t gres_ctld_step_test(List step_gres_list, List job_gres_list,
+				    int node_offset, bool first_step_node,
+				    uint16_t cpus_per_task, int max_rem_nodes,
+				    bool ignore_alloc,
+				    uint32_t job_id, uint32_t step_id,
+				    bool test_mem,
+				    job_resources_t *job_resrcs_ptr,
+				    int *err_code);
+
+/*
+ * If a step gres request used gres_per_step it must be tested more than just in
+ * gres_ctld_step_test. This function only acts when gres_per_step is used
+ * IN step_gres_list  - step's requested GRES data structure
+ * IN job_ptr - Job data
+ * IN/OUT nodes_avail - Bitstring of nodes available for this step to use
+ * IN min_nodes - minimum nodes required for this step
+ */
+extern void gres_ctld_step_test_per_step(List step_gres_list,
+					 job_record_t *job_ptr,
+					 bitstr_t *nodes_avail,
+					 int min_nodes);
+
 /*
  * Allocate resource to a step and update job and step gres information
  * IN step_gres_list - step's gres_list built by
