@@ -40,6 +40,8 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE	/* for setresuid() */
+
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -49,6 +51,7 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <unistd.h>
 
 #include "src/common/bitstring.h"
 #include "src/common/fd.h"
@@ -78,6 +81,8 @@ bitstr_t *trigger_down_nodes_bitmap = NULL;
 bitstr_t *trigger_drained_nodes_bitmap = NULL;
 bitstr_t *trigger_fail_nodes_bitmap = NULL;
 bitstr_t *trigger_up_nodes_bitmap   = NULL;
+bitstr_t *trigger_draining_nodes_bitmap = NULL;
+bitstr_t *trigger_resume_nodes_bitmap = NULL;
 static bool trigger_bb_error = false;
 static bool trigger_node_reconfig = false;
 static bool trigger_pri_ctld_fail = false;
@@ -529,53 +534,67 @@ extern void trigger_front_end_up(front_end_record_t *front_end_ptr)
 
 extern void trigger_node_down(node_record_t *node_ptr)
 {
-	int inx = node_ptr - node_record_table_ptr;
-
 	xassert(verify_lock(NODE_LOCK, READ_LOCK));
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_down_nodes_bitmap == NULL)
 		trigger_down_nodes_bitmap = bit_alloc(node_record_count);
-	bit_set(trigger_down_nodes_bitmap, inx);
+	bit_set(trigger_down_nodes_bitmap, node_ptr->index);
 	slurm_mutex_unlock(&trigger_mutex);
 }
 
 extern void trigger_node_drained(node_record_t *node_ptr)
 {
-	int inx = node_ptr - node_record_table_ptr;
-
 	xassert(verify_lock(NODE_LOCK, READ_LOCK));
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_drained_nodes_bitmap == NULL)
 		trigger_drained_nodes_bitmap = bit_alloc(node_record_count);
-	bit_set(trigger_drained_nodes_bitmap, inx);
+	bit_set(trigger_drained_nodes_bitmap, node_ptr->index);
 	slurm_mutex_unlock(&trigger_mutex);
 }
 
 extern void trigger_node_failing(node_record_t *node_ptr)
 {
-	int inx = node_ptr - node_record_table_ptr;
-
 	xassert(verify_lock(NODE_LOCK, READ_LOCK));
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_fail_nodes_bitmap == NULL)
 		trigger_fail_nodes_bitmap = bit_alloc(node_record_count);
-	bit_set(trigger_fail_nodes_bitmap, inx);
+	bit_set(trigger_fail_nodes_bitmap, node_ptr->index);
 	slurm_mutex_unlock(&trigger_mutex);
 }
 
 extern void trigger_node_up(node_record_t *node_ptr)
 {
-	int inx = node_ptr - node_record_table_ptr;
-
 	xassert(verify_lock(NODE_LOCK, READ_LOCK));
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_up_nodes_bitmap == NULL)
 		trigger_up_nodes_bitmap = bit_alloc(node_record_count);
-	bit_set(trigger_up_nodes_bitmap, inx);
+	bit_set(trigger_up_nodes_bitmap, node_ptr->index);
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_node_draining(node_record_t *node_ptr)
+{
+	xassert(verify_lock(NODE_LOCK, READ_LOCK));
+
+	slurm_mutex_lock(&trigger_mutex);
+	if (!trigger_draining_nodes_bitmap)
+		trigger_draining_nodes_bitmap = bit_alloc(node_record_count);
+	bit_set(trigger_draining_nodes_bitmap, node_ptr->index);
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_node_resume(node_record_t *node_ptr)
+{
+	xassert(verify_lock(NODE_LOCK, READ_LOCK));
+
+	slurm_mutex_lock(&trigger_mutex);
+	if (!trigger_resume_nodes_bitmap)
+		trigger_resume_nodes_bitmap = bit_alloc(node_record_count);
+	bit_set(trigger_resume_nodes_bitmap, node_ptr->index);
 	slurm_mutex_unlock(&trigger_mutex);
 }
 
@@ -587,23 +606,23 @@ extern void trigger_reconfig(void)
 	slurm_mutex_lock(&trigger_mutex);
 	trigger_node_reconfig = true;
 	if (trigger_down_front_end_bitmap)
-		trigger_down_front_end_bitmap = bit_realloc(
-			trigger_down_front_end_bitmap, node_record_count);
+		bit_realloc(trigger_down_front_end_bitmap, node_record_count);
 	if (trigger_up_front_end_bitmap)
-		trigger_up_front_end_bitmap = bit_realloc(
-			trigger_up_front_end_bitmap, node_record_count);
+		bit_realloc(trigger_up_front_end_bitmap, node_record_count);
 	if (trigger_down_nodes_bitmap)
-		trigger_down_nodes_bitmap = bit_realloc(
-			trigger_down_nodes_bitmap, node_record_count);
+		bit_realloc(trigger_down_nodes_bitmap, node_record_count);
 	if (trigger_drained_nodes_bitmap)
-		trigger_drained_nodes_bitmap = bit_realloc(
-			trigger_drained_nodes_bitmap, node_record_count);
+		bit_realloc(trigger_drained_nodes_bitmap, node_record_count);
 	if (trigger_fail_nodes_bitmap)
-		trigger_fail_nodes_bitmap = bit_realloc(
-			trigger_fail_nodes_bitmap, node_record_count);
+		bit_realloc(trigger_fail_nodes_bitmap, node_record_count);
 	if (trigger_up_nodes_bitmap)
-		trigger_up_nodes_bitmap = bit_realloc(
-			trigger_up_nodes_bitmap, node_record_count);
+		bit_realloc(trigger_up_nodes_bitmap, node_record_count);
+	if (trigger_draining_nodes_bitmap)
+		trigger_draining_nodes_bitmap = bit_realloc(
+			trigger_draining_nodes_bitmap, node_record_count);
+	if (trigger_resume_nodes_bitmap)
+		trigger_resume_nodes_bitmap = bit_realloc(
+			trigger_resume_nodes_bitmap, node_record_count);
 	slurm_mutex_unlock(&trigger_mutex);
 	unlock_slurmctld(node_read_lock);
 }
@@ -890,7 +909,7 @@ extern int trigger_state_save(void)
 	xfree(reg_file);
 	xfree(new_file);
 	unlock_state_files();
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return error_code;
 }
 
@@ -948,7 +967,7 @@ extern void trigger_state_restore(void)
 		error("Can't recover trigger state, data version "
 		      "incompatible");
 		xfree(ver_str);
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		return;
 	}
 	xfree(ver_str);
@@ -969,7 +988,7 @@ unpack_error:
 		fatal("Incomplete trigger data checkpoint file, start with '-i' to ignore this. Warning: using -i will lose the data that can't be recovered.");
 	error("Incomplete trigger data checkpoint file");
 fini:	verbose("State of %d triggers recovered", trigger_cnt);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 }
 
 static bool _front_end_job_test(bitstr_t *front_end_bitmap,
@@ -1226,15 +1245,15 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		 * nodes have been idle for at least the offset time */
 		time_t min_idle = now - (trig_in->trig_time - 0x8000);
 		int i;
-		node_record_t *node_ptr = node_record_table_ptr;
+		node_record_t *node_ptr;
 		bitstr_t *trigger_idle_node_bitmap;
 
 		trigger_idle_node_bitmap = bit_alloc(node_record_count);
-		for (i = 0; i < node_record_count; i++, node_ptr++) {
+		for (i = 0; (node_ptr = next_node(&i)); i++) {
 			if (!IS_NODE_IDLE(node_ptr) ||
 			    (node_ptr->last_busy > min_idle))
 				continue;
-			bit_set(trigger_idle_node_bitmap, i);
+			bit_set(trigger_idle_node_bitmap, node_ptr->index);
 		}
 		if (trig_in->nodes_bitmap == NULL) {    /* all nodes */
 			xfree(trig_in->res_id);
@@ -1285,6 +1304,58 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		}
 	}
 
+	if ((trig_in->trig_type & TRIGGER_TYPE_DRAINING) &&
+	    trigger_draining_nodes_bitmap &&
+	    (bit_ffs(trigger_draining_nodes_bitmap) != -1)) {
+		if (!trig_in->nodes_bitmap) { /* all nodes */
+			xfree(trig_in->res_id);
+			trig_in->res_id =
+				bitmap2node_name(trigger_draining_nodes_bitmap);
+			trig_in->state = 1;
+		} else if (bit_overlap_any(trig_in->nodes_bitmap,
+					   trigger_draining_nodes_bitmap)) {
+			bit_and(trig_in->nodes_bitmap,
+				trigger_draining_nodes_bitmap);
+			xfree(trig_in->res_id);
+			trig_in->res_id =
+				bitmap2node_name(trig_in->nodes_bitmap);
+			trig_in->state = 1;
+		}
+		if (trig_in->state == 1) {
+			trig_in->trig_time =
+				now + (trig_in->trig_time - 0x8000);
+			log_flag(TRIGGERS, "trigger[%u] for node %s draining",
+				 trig_in->trig_id, trig_in->res_id);
+			return;
+		}
+	}
+
+	if ((trig_in->trig_type & TRIGGER_TYPE_RESUME) &&
+	    trigger_resume_nodes_bitmap &&
+	    (bit_ffs(trigger_resume_nodes_bitmap) != -1)) {
+		if (!trig_in->nodes_bitmap) { /* all nodes */
+			xfree(trig_in->res_id);
+			trig_in->res_id =
+				bitmap2node_name(trigger_resume_nodes_bitmap);
+			trig_in->state = 1;
+		} else if (bit_overlap_any(trig_in->nodes_bitmap,
+					   trigger_resume_nodes_bitmap)) {
+			bit_and(trig_in->nodes_bitmap,
+				trigger_resume_nodes_bitmap);
+			xfree(trig_in->res_id);
+			trig_in->res_id =
+				bitmap2node_name(trig_in->nodes_bitmap);
+			trig_in->state = 1;
+		}
+		if (trig_in->state == 1) {
+			trig_in->trig_time =
+				now + (trig_in->trig_time - 0x8000);
+			log_flag(TRIGGERS, "trigger[%u] for node %s resume",
+				 trig_in->trig_id, trig_in->res_id);
+			return;
+		}
+	}
+
 	if ((trig_in->trig_type & TRIGGER_TYPE_RECONFIG) &&
 	    trigger_node_reconfig) {
 		trig_in->state = 1;
@@ -1470,7 +1541,6 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 		trig_in->child_pid = child_pid;
 	} else if (child_pid == 0) {
 		bool run_as_self = (uid == slurm_conf.slurm_user_id);
-
 		closeall(0);
 		setpgid(0, 0);
 		setsid();
@@ -1482,8 +1552,8 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 			error("trigger: setgid: %m");
 			exit(1);
 		}
-		if ((setuid(uid) == -1) && !run_as_self) {
-			error("trigger: setuid: %m");
+		if ((setresuid(uid, uid, -1) == -1) && !run_as_self) {
+			error("trigger: setresuid: %m");
 			exit(1);
 		}
 		execv(program, args);
@@ -1498,26 +1568,20 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 
 static void _clear_event_triggers(void)
 {
-	if (trigger_down_front_end_bitmap) {
-		bit_nclear(trigger_down_front_end_bitmap,
-			   0, (bit_size(trigger_down_front_end_bitmap) - 1));
-	}
-	if (trigger_up_front_end_bitmap) {
-		bit_nclear(trigger_up_front_end_bitmap,
-			   0, (bit_size(trigger_up_front_end_bitmap) - 1));
-	}
-	if (trigger_down_nodes_bitmap) {
-		bit_nclear(trigger_down_nodes_bitmap,
-			   0, (bit_size(trigger_down_nodes_bitmap) - 1));
-	}
-	if (trigger_drained_nodes_bitmap) {
-		bit_nclear(trigger_drained_nodes_bitmap,
-			   0, (bit_size(trigger_drained_nodes_bitmap) - 1));
-	}
-	if (trigger_up_nodes_bitmap) {
-		bit_nclear(trigger_up_nodes_bitmap,
-			   0, (bit_size(trigger_up_nodes_bitmap) - 1));
-	}
+	if (trigger_down_front_end_bitmap)
+		bit_clear_all(trigger_down_front_end_bitmap);
+	if (trigger_up_front_end_bitmap)
+		bit_clear_all(trigger_up_front_end_bitmap);
+	if (trigger_down_nodes_bitmap)
+		bit_clear_all(trigger_down_nodes_bitmap);
+	if (trigger_drained_nodes_bitmap)
+		bit_clear_all(trigger_drained_nodes_bitmap);
+	if (trigger_up_nodes_bitmap)
+		bit_clear_all(trigger_up_nodes_bitmap);
+	if (trigger_draining_nodes_bitmap)
+		bit_clear_all(trigger_draining_nodes_bitmap);
+	if (trigger_resume_nodes_bitmap)
+		bit_clear_all(trigger_resume_nodes_bitmap);
 	trigger_node_reconfig = false;
 	trigger_bb_error = false;
 	trigger_pri_ctld_fail = false;
@@ -1669,4 +1733,6 @@ extern void trigger_fini(void)
 	FREE_NULL_BITMAP(trigger_drained_nodes_bitmap);
 	FREE_NULL_BITMAP(trigger_fail_nodes_bitmap);
 	FREE_NULL_BITMAP(trigger_up_nodes_bitmap);
+	FREE_NULL_BITMAP(trigger_draining_nodes_bitmap);
+	FREE_NULL_BITMAP(trigger_resume_nodes_bitmap);
 }
