@@ -58,8 +58,8 @@
 #include "src/common/log.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_accounting_storage.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/slurm_time.h"
 #include "src/common/uid.h"
@@ -67,6 +67,8 @@
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/hash.h"
+
 #include "src/slurmdbd/read_config.h"
 #include "src/slurmdbd/rpc_mgr.h"
 #include "src/slurmdbd/proc_req.h"
@@ -160,6 +162,9 @@ int main(int argc, char **argv)
 	if (slurm_auth_init(NULL) != SLURM_SUCCESS) {
 		fatal("Unable to initialize authentication plugins");
 	}
+	if (hash_g_init() != SLURM_SUCCESS) {
+		fatal("failed to initialize hash plugin");
+	}
 	if (slurm_acct_storage_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize %s accounting storage plugin",
 		      slurm_conf.accounting_storage_type);
@@ -979,7 +984,7 @@ static void _become_slurm_user(void)
 	}
 }
 
-extern void _restart_self(int argc, char **argv)
+static void _restart_self(int argc, char **argv)
 {
 	info("Restarting self");
 	if (execvp(argv[0], argv))
